from __future__ import division

import os
import pytest
import numpy as np
from numpy.testing import assert_array_equal, assert_allclose
import pyamps

def test_model_vectors():

    fn = os.path.abspath(os.path.join(pyamps.model_utils.basepath, 'coefficients','test_model.txt'))

    model_vectors = pyamps.model_utils.get_model_vectors(v=0, By=0, Bz=1, tilt=0.5, f107=0.3, coeff_fn = fn)
    # const,cosca,tilt and f107 terms non-zero
    tor_c, tor_s, pol_c, pol_s, pol_index, tor_index = model_vectors

    assert_allclose(tor_c.flatten(), [-0.186221  , -0.18158214, -0.14932639,  0.15199847, -0.30006201])
    assert_allclose(tor_s.flatten(), [ 0.        ,  1.21234145,  0.        , -0.64112046,  0.12969112])  # tor_s_const = 0
    assert_allclose(pol_c.flatten(), [ 0.87205983, -0.37122322, -0.07961519, -0.50883854,  0.09931672])
    assert_allclose(pol_s.flatten(), [ 0.        ,  0.36922202,  0.        ,  0.29350035, -0.22020463])
    assert tor_index[4] == (2, 2)
    assert pol_index[2] == (2, 0)
    assert len(tor_index) == 5



# import swipe
import numpy as np
import os
from pyswipe.swipe import SWIPE

plotdir = '/plotdir/'

import matplotlib as mpl
mplBkgrnd = 'QtAgg'
mpl.use(mplBkgrnd)

# MPL opts

mpl.rcParams.update({'text.color': 'k'})
mpl.rcParams.update({'axes.labelcolor': 'k'})
mpl.rcParams.update({'xtick.color': 'k'})
mpl.rcParams.update({'ytick.color': 'k'})
mpl.rcParams.update({'font.size': 15})
# mpl.rcParams.update({'font.family': 'sans-serif'})
# mpl.rcParams.update({'font.sans-serif': 'Arial'})
mpl.rcParams.update({'text.usetex': False})

mpl.rcParams.update({'figure.figsize': [10.0, 8.0]})
mpl.rcParams.update({'savefig.directory': plotdir})

import matplotlib.pyplot as plt
plt.ion()

##############################
# Data from other figures

# clock angles
thetac = np.array([0, 45, 90, 135, 180, 225, 270, 315, 360])
thetacrad = np.array([0, 1/4, 1/2, 3/4, 1, 5/4, 3/2, 7/4, 2]) * np.pi

#Förster and Haaland (2015) Figures 2 (NH) and 3 (SH)
# They simply group all available data by IMF clock angle sector, and ensure that the bias vector length (see Section 3 of Haaland et al, 2007, for description) is less than 0.96
# They could not screen for seasonal dependence, since Cluster coverage varies with season
fhn = np.array([16.7, 28.2, 45.1, 61.0, 70.6, 63.7, 42.4, 22.9, 16.7])
fhs = np.array([16.1, 24.9, 42.8, 60.5, 75.0, 67.0, 42.8, 28.0, 16.1])


# Cousins and Shepherd (2010) Figure 3
# 2.2 < E_{sw} < 2.9 mV/m$, where $E_{sw} = |V_x B_T|$ is the "solar wind electric field magnitude."
# (Can get E_sw = 2.5 mV/m via for example V = 500 km/s and B_T = 5 nT, 
# NH for tilt > 10deg and in the SH for tilt < -10deg (SH second number)
cousinsn = np.array([20, 30, 40, 53, 67, 65, 50, 38, 20])
cousinss = np.array([11, 20, 37, 54, 71, 53, 37, 23, 11])


# Weimer (2005) Figure 2
# B_T = 5 nT
# V = 450 km/s
#N_sw = 4.0/cc
#tilt = 0 deg
weimer = np.array([24, 40, 69, 93, 102, 90, 58, 32, 24])


# Papitashvili and Rich (2002) Figure 2
#B_T = 5 nT
#n = 5 cm^{-3},
#V = 400 km/s.
#Dipole tilt is approximately zero, since they use all data centered around each equinox in a two-month window. NH and SH are given as first and second numbers respectively:
papitashn = np.array([11, 22, 40, 69, 79, 65, 33, 19, 11])
papitashs = np.array([14, 18, 33, 69, 88, 78, 45, 28, 14])

#For our comparison, we use $V_x = 450$~m/s and $B_T = 5$~nT, which corresponds to $E_{sw} = 2.25$~mV/m.

bt = 5                          # nT
#thetas = thetacrad
thetacraddeets = np.arange(0,2.01,0.025) * np.pi

bzs = bt * np.cos(thetacraddeets)
bys = bt * np.sin(thetacraddeets)
dptilt = 0.
# dptilts = np.arange(-33,33.1,11.)
# dptilts = np.arange(-22,22.1,11.)
# BY, DP = np.meshgrid(bys,dptilts,indexing='ij')

swvel = 450
#swden  = 5.590                  # NOTE: SWIPE does not use swdensity
# dptilt = -1.0
f107val = 120

savefig = True

##############################

zero_latNU = None

minlat = 47
minlat_for_cpcp_calc = 60

dPhidn = dict()
dPhids = dict()
locdn = dict()
locds = dict()
for i in range(1):

    # lists for holding CPCP for each bz value
    dPhiNss, dPhiSss = [], []
    locnss, locsss = [], []
    for j,(by,bz) in enumerate(zip(bys,bzs)):
    
        print(f"i={j}, by={by:.2f}, bz={bz:.2f}")
    
        # lists for holding CPCP in each hemisphere and the locations of extrema
        dPhiNs, dPhiSs = [], []
        locns, locss = [], []
    
        # paramstr = f"$B_y$={by}, $B_z$={bz},"+" $v_{SW}$="+f"{swvel}"
        # tottitl = paramstr + titls[i]
    
        # paramstr = f"$B_y$={by}, $B_z$={bz}, $\Psi$={tilt}$^\circ$,F10.7={f107val}\n"
        # print(f"v = {v} km/s, By = {by} nT, Bz = {bz} nT, Psi = {tilt} deg, F10.7 = {f107}")
    
        if j == 0:
            this = SWIPE(-swvel,by,bz,dptilt,f107val,
                         minlat=minlat,
                         zero_lats=zero_latNU)
    
            mlats, mlts = this.scalargrid
            mlatv, mltv = this.vectorgrid
            mlatvn, mlatvs = np.split(mlatv, 2)
            mltvn, mltvs = np.split(mltv, 2)
            mlatsn, mlatss = np.split(mlats, 2)
            mltsn, mltss = np.split(mlts, 2)
    
        else:
            this.update_model(-swvel, by, bz, dptilt, f107val)
    
        phin, phis = np.split(this.get_potential(), 2)
        phin = phin - np.median(phin)
        phis = phis - np.median(phis)
    
        OKindsn, OKindss = np.split(np.abs(mlats).flatten() >= minlat_for_cpcp_calc,2)
        # mlatsn, mlatss = np.split(mlats,2)
        # mltsn, mltss = np.split(mlts,2)
        # cpcpmlatsn,cpcpmltsn = mlatsn.flatten()[OKindsn],mltsn.flatten()[OKindsn]
        # cpcpmlatss,cpcpmltss = mlatss.flatten()[OKindss],mltss.flatten()[OKindss]
        dPhiN = phin[OKindsn].max()-phin[OKindsn].min()
        dPhiS = phis[OKindss].max()-phis[OKindss].min()
        minn,maxn = np.argmin(phin[OKindsn]),np.argmax(phin[OKindsn])
        mins,maxs = np.argmin(phis[OKindss]),np.argmax(phis[OKindss])
        
        dPhiNs.append(dPhiN)
        dPhiSs.append(dPhiS)
        locns.append([minn,maxn])
        locss.append([mins,maxs])
    
        dPhiNs = np.array(dPhiNs)
        dPhiSs = np.array(dPhiSs)
    
        # dPhiNs = dPhiNs.reshape(BY.shape)
        # dPhiSs = dPhiSs.reshape(BY.shape)
    
        locns, locss = np.array(locns), np.array(locss)
    
        dPhiNss.append(dPhiNs)
        dPhiSss.append(dPhiSs)
        locnss.append(locns)
        locsss.append(locns)
    
    dPhidn[i] = dPhiNss
    dPhids[i] = dPhiSss
    
    locdn[i] = locnss
    locds[i] = locsss
    
cpcpn = np.array(dPhidn[0]).ravel()
cpcps = np.array(dPhids[0]).ravel()


# REAL THING
cpcp = [cpcpn, cpcps]
fh = [fhn, fhs]
papitash = [papitashn,papitashs]
cousins = [cousinsn,cousinss]
weimer = weimer
titles = ['North','South']

weimerdeets = np.interp(thetacraddeets, thetacrad, weimer)
fhdeets = [np.interp(thetacraddeets, thetacrad, fhh) for fhh in fh]
papitashdeets = [np.interp(thetacraddeets, thetacrad, papitashv) for papitashv in papitash]
papitashdeets = [np.interp(thetacraddeets, thetacrad, papitashv) for papitashv in papitash]
cousinsdeets = [np.interp(thetacraddeets, thetacrad, cousinz) for cousinz in cousins]

showdeets = True
showdots = True and showdeets
showdotsize = 20
if showdeets:
    guestthetarad = thetacraddeets
    weimershow = weimerdeets
    fhshow = fhdeets
    papitashow = papitashdeets
    cousinshow = cousinsdeets
else:
    guestthetarad = thetacrad
    weimershow = weimer
    fhshow = fh
    papitashow = papitash
    cousinshow = cousins

fig = plt.figure(3,figsize=(14,7))
plt.clf()
ax0 = plt.subplot(121,projection='polar')
ax1 = plt.subplot(122,projection='polar')
rticks = [30, 60, 90]
rticks = [50,100]

weimercolor = 'black'
wkey = dict(color=weimercolor, linestyle='-',linewidth=1.5)
fhkey = dict(linestyle='-.',linewidth=2.2)
prkey = dict(linestyle='--', linewidth=1.5)
cskey = dict(linestyle=':', linewidth=2)

    
for i,ax in enumerate([ax0,ax1]):
    ax.set_theta_zero_location("N")  # theta=0 at the top
    ax.set_theta_direction(-1)  # theta increasing clockwise
    ax.set_rlabel_position(67.5)  # Move radial labels away from legend
    ax.set_rlabel_position(45)  # Move radial labels away from legend
    ax.spines['polar'].set_visible(False)

    # ax.set_rticks(rticks,labels=[f"{num} kV" for num in rticks])  # Less radial ticks
    ax.set_rticks(np.array(rticks),labels=[f"{num} kV" for num in rticks])  # Less radial ticks

    ax.set_xticks(np.array([-1/2, 0, 1/2, 1])*np.pi, labels=["270$^\circ$", r"$\theta_c = 0^\circ$", f"90$^\circ$", "180$^\circ$"])
    ax.set_thetalim(-np.pi, np.pi)

    ax.set_title(titles[i])
    
    nalpha = 0.1 if i == 1 else 1.0
    salpha = 0.1 if i == 0 else 1.0

    ncolor = 'blue'
    scolor = 'red'

    ncolor = 'blue' if i == 0 else 'black'
    scolor = 'red' if i == 1 else 'black'

    # ORIG LEGEND STUFF
    hicnlabel = 'Swarm Hi-C'
    pr02nlabel = 'PR02'
    cs10nlabel = 'CS10'
    fh15nlabel = 'FH15'
    wlabel = 'W05'

    hicslabel = None
    pr02slabel = None
    cs10slabel = None
    fh15slabel = None

    # NEW LEGEND STUFF
    if i == 0:
        # hicnlabel = 'Hi-C'
        # pr02nlabel = 'PR02'
        # cs10nlabel = 'CS10'
        # fh15nlabel = 'FH15'
        # wlabel = 'W05'
    
        hicnlabel = 'Hi-C (this study)'
        pr02nlabel = 'P&R (2002)'
        cs10nlabel = 'C&S (2010)'
        fh15nlabel = 'F&H (2015)'
        wlabel = 'Weimer (2005)'

        hicnlabel = 'Hi-C (this study)'
        pr02nlabel = 'PR02'
        cs10nlabel = 'CS10'
        fh15nlabel = 'FH15'
        wlabel = 'W05'

        hicslabel = None
        pr02slabel = None
        cs10slabel = None
    elif i == 1:
        hicnlabel = None
        pr02nlabel = None
        fh15nlabel = None
        cs10nlabel = None
        wlabel = 'W05'
    
        hicslabel = 'Hi-C'
        pr02slabel = 'PR02'
        fh15slabel = 'FH15'
        cs10slabel = 'CS10'
    

    # Cousins and Shepherd (2010)
    ax.plot(guestthetarad, cousinshow[0],label=cs10nlabel, **cskey, color=ncolor, alpha=nalpha)
    ax.plot(guestthetarad, cousinshow[1],label=cs10slabel, **cskey, color=scolor , alpha=salpha)
    if showdots:
        ax.scatter(thetacrad, cousins[0], s=showdotsize, color=ncolor, alpha=nalpha)
        ax.scatter(thetacrad, cousins[1], s=showdotsize, color=scolor , alpha=salpha)

    # Förster and Haaland (2015)
    ax.plot(guestthetarad, fhshow[0],label=fh15nlabel, **fhkey, color=ncolor, alpha=nalpha)
    ax.plot(guestthetarad, fhshow[1],label=fh15slabel, **fhkey, color=scolor , alpha=salpha)
    if showdots:
        ax.scatter(thetacrad, fh[0], s=showdotsize, color=ncolor, alpha=nalpha)
        ax.scatter(thetacrad, fh[1], s=showdotsize, color=scolor , alpha=salpha)

    # Papitashvili and Rich (2002)
    ax.plot(guestthetarad, papitashow[0],label=pr02nlabel, **prkey, color=ncolor, alpha=nalpha)
    ax.plot(guestthetarad, papitashow[1],label=pr02slabel, **prkey, color=scolor , alpha=salpha)
    if showdots:
        ax.scatter(thetacrad, papitash[0], s=showdotsize, color=ncolor, alpha=nalpha)
        ax.scatter(thetacrad, papitash[1], s=showdotsize, color=scolor , alpha=salpha)
    
    # Swarm Hi-C (this study, Hatch et al)
    ax.plot(thetacraddeets, cpcp[0], color=ncolor,label=hicnlabel,linewidth=4,
            alpha = nalpha)
    ax.plot(thetacraddeets, cpcp[1], color=scolor,label=hicslabel,linewidth=4,
            alpha = salpha)
    
    # Weimer (2005)
    ax.plot(guestthetarad, weimershow, label='W05', **wkey)
    if showdots:
        ax.scatter(thetacrad, weimer, s=showdotsize, color=weimercolor)
    
    
    # if i == 0:
    #     ax.legend(loc='upper center')
    
    if i == 0:
        ax.legend(loc='upper center')

    # ax.set_rmax(2)
    # ax.set_rticks([0.5, 1, 1.5, 2])  # Less radial ticks
    # ax.set_rlabel_position(-22.5)  # Move radial labels away from plotted line
    ax.grid(True)
    
plt.tight_layout()


plt.savefig(plotdir+'CPCP_comparison2.png',dpi=300)



"""
Some examples showing how to use the dask-based Swipe functions that are good at computing time series of model outputs along, say, satellite tracks.
"""

from pyswipe.swipe import SWIPE, get_E, get_v, get_pflux
from datetime import datetime

import numpy as np
from apexpy import Apex


vsw = 300 # solar wind velocity in km/s 
By, Bz = -4, -3 # IMF By and Bz in nT
tilt, f107 = 20, 150 # dipole tilt angle in degrees, F10.7 index in s.f.u

h_R = 110.
refdt = datetime(2015,6,1,0,0)

m = SWIPE(vsw, 
          By, 
          Bz, 
          tilt, 
          f107)

alat, mlt = m.vectorgrid
alat, mlt = map(lambda x: x.flatten(), [alat, mlt])

a = Apex(date=refdt,refh=h_R)

alon = a.mlt2mlon(mlt, refdt)
height = np.ones_like(alon)*300.

glat, glon, error = a.apex2geo(alat, alon, height)
time = [refdt]*height.size

# Convert inputs to arrays, which is what dask functions get_E, get_v, and get_pflux are expecting
vsw, By, Bz, tilt, f107 = map(lambda x: np.ones_like(glat)*x, [vsw, By, Bz, tilt, f107])

epoch = a.year

chunksize = 15000               # chunksize to be used by dask

f1, f2, f3, g1, g2, g3, d1, d2, d3, e1, e2, e3 = a.basevectors_apex(alat, alon, height, coords  = 'apex')

## Compare E-field
E_e, E_n, E_u = get_E(glat, glon, height, time, vsw, By, Bz, tilt, f107,
                      coords='geo',
                      epoch = epoch, h_R = h_R, chunksize = chunksize)

E_d1alt, E_d2alt = get_E(glat, glon, height, time, vsw, By, Bz, tilt, f107,
                   coords='apex',
                   epoch = epoch, h_R = h_R, chunksize = chunksize)


E_d1, E_d2 = m.get_efield_MA(alat,mlt)

E_ealt = E_d1*d1[0] + E_d2*d2[0]
E_nalt = E_d1*d1[1] + E_d2*d2[1]
E_ualt = E_d1*d1[2] + E_d2*d2[2]

print("")
print("Comparing E-field from get_E and SWIPE.get_efield_MA in geo coords")
print("E_e median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((E_ealt-E_e)/E_e*100,3)))))
print("E_n median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((E_nalt-E_n)/E_n*100,3)))))
print("E_u median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((E_ualt-E_u)/E_u*100,3)))))
print("")
print("Comparing E-field from get_E and SWIPE.get_efield_MA in apex coords")
print("E_d1 median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((E_d1alt-E_d1)/E_d1*100,3)))))
print("E_d2 median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((E_d2alt-E_d2)/E_d2*100,3)))))
print("")

## Compare v
v_e, v_n, v_u = get_v(glat, glon, height, time, vsw, By, Bz, tilt, f107,
                           epoch = epoch, h_R = h_R,
                           coords = 'geo',
                           chunksize = chunksize)

v_e1alt, v_e2alt = get_v(glat, glon, height, time, vsw, By, Bz, tilt, f107,
                           epoch = epoch, h_R = h_R,
                           coords = 'apex',
                           chunksize = chunksize)


v_e1, v_e2 = m.get_convection_vel_MA(alat,mlt)

v_ealt = v_e1*e1[0] + v_e2*e2[0]
v_nalt = v_e1*e1[1] + v_e2*e2[1]
v_ualt = v_e1*e1[2] + v_e2*e2[2]


print("")
print("Comparing convection from get_v and SWIPE.get_convection_vel_MA in geo coords")
print("v_e median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((v_ealt-v_e)/v_e*100,3)))))
print("v_n median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((v_nalt-v_n)/v_n*100,3)))))
print("v_u median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((v_ualt-v_u)/v_u*100,3)))))
print("")
print("Comparing convection from get_v and SWIPE.get_convection_vel_MA in apex coords")
print("v_e1 median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((v_e1alt-v_e1)/v_e1*100,3)))))
print("v_e2 median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((v_e2alt-v_e2)/v_e2*100,3)))))
print("")


## Compare pflux
pfluxe1, pfluxe2, pfluxpar = m.get_poynting_flux(mlat = alat, mlt = mlt,
                                               times = time,
                                               heights = height,
                                               apex_refdate=refdt,
                                               apex_refheight=h_R,
                                               grid = False,
                                               killpoloidalB=True)

pflux = get_pflux(glat, glon, height, time, vsw, By, Bz, tilt, f107,
                  epoch = epoch, h_R = h_R,
                  coords = 'apex',
                  chunksize = chunksize)

print("")
print("Comparing field-aligned poynting flux from get_pflux and SWIPE.get_poynting_flux in apex coords")
# print("v_e median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((v_ealt-v_e)/v_e*100,3)))))
# print("v_n median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((v_nalt-v_n)/v_n*100,3)))))
print("pflux_parallel median abs % diff: {:.2f}%".format(np.median(np.abs(np.round((pflux[2]-pfluxpar)/pfluxpar*100,3)))))
print("")




import numpy as np

from pyswipe.swipe import SWIPE

plotdir = '/plotdir/'

import matplotlib as mpl
# MPL opts
mpl.rcParams.update({'text.color': 'k'})
mpl.rcParams.update({'axes.labelcolor': 'k'})
mpl.rcParams.update({'xtick.color': 'k'})
mpl.rcParams.update({'ytick.color': 'k'})
mpl.rcParams.update({'text.usetex': False})

mpl.rcParams.update({'figure.figsize': [10.0, 8.0]})
mpl.rcParams.update({'savefig.directory': plotdir})

import matplotlib.pyplot as plt
plt.ion()

from polplot.polplot import Polarplot

d2r = np.pi/180

dontbescaredofnegs = True

do_jdote_screen = True
min_jdote = 0.5

do_hall_screen = True
min_hall = 0.05
max_hall = 100.

##############################

conductance = 'h'

assert conductance in ['p','h','c']

if conductance == 'p':
    titstr = 'Pedersen'
elif conductance == 'h':
    titstr = 'Hall'
elif conductance == 'c':
    titstr = 'Cowling'

B = 5                          # nT

ca = {(0, 1): 0, (0, 0):-45, (1, 0):-90,(2,0):-135,(2,1):180,(0,2):45,(1,2):90,(2,2):135}

tilts = [-25, 0, 25]

v = 450
f107val = 120

minlat = 45

potlevels = np.arange(0, 8.1, 2)
cmap = plt.cm.Blues
extend = 'both'
if dontbescaredofnegs:
    # potlevels = np.arange(-8, 8, 2)+1
    # cmap = plt.cm.bwr
    potlevels = np.arange(-18, 18, 2)+1
    cmap = plt.cm.bwr
    extend = None

doCalcBhattacharyya = False      # Code for this taken from "pct_diff_ns_currents_for_varying_vswB_and_clock_angle.py"


zero_latNU = None

##############################
# Initialize models
swipe_n = SWIPE(v, B, B, 0, f107val,
             minlat=minlat)#,
print(1/0)
swipe_s = SWIPE(v, B, B, 0, f107val,
             minlat=minlat)#,

##############################
# Get grid for calculating conductances
from pyswipe.plot_utils import equal_area_grid, Polarplot, get_h2d_bin_areas

# defaults
# dr = 2
# N = 20
# M0 = 4

# dad
dr = 1
N = 40
M0 = 4

maxlat = swipe_n.maxlat

grid = equal_area_grid(dr = dr, M0 = M0, N = N)

mlat, mlt = grid[0], grid[1]

mltc  = grid[1] + grid[2]/2. # shift to the center points of the bins
mlatc = grid[0] + dr/2  # shift to the center points of the bins

mltc  = mltc[ (mlatc >= minlat) & (mlatc <= maxlat)]# & (mlat <=60 )]
mlatc = mlatc[(mlatc >= minlat) & (mlatc <= maxlat)]# & (mlat <= 60)]

mlatc = np.hstack((mlatc, -mlatc)) # add southern hemisphere points
mltc  = np.hstack((mltc ,  mltc)) # add southern hemisphere points

mlat = np.hstack((mlat, -mlat)) # add southern hemisphere points
mlt  = np.hstack((mlt ,  mlt)) # add southern hemisphere points

mltres = grid[2]
mltres = np.hstack((mltres, mltres))

mlat_n, mlt_n = map(lambda x:np.split(x, 2)[0], (mlat, mlt))
mlat_s, mlt_s = map(lambda x:np.split(x, 2)[1], (mlat, mlt))

zero_latNU = None

# if doCalcBhattacharyya:

# Ripped from pyamps.amps.get_integrated_upward_current
REFRE = 6371.2 # Reference radius used in geomagnetic modeling

# get surface area element in each cell:
mlat, mlt = swipe_n.scalargrid
mlt_sorted = np.sort(np.unique(mlt))
mltresr = (mlt_sorted[1] - mlt_sorted[0]) * np.pi / 12
mlat_sorted = np.sort(np.unique(mlat))
mlatres = (mlat_sorted[1] - mlat_sorted[0]) * np.pi / 180
R = (REFRE + swipe_n.height) * 1e3  # radius in meters
dS = R**2 * np.cos(mlat * np.pi/180) * mlatres * mltresr

dS_n,dS_s = np.split(dS, 2)
assert np.array_equal(dS_n,dS_s),"Uh oh, calculations later assume dS_n == dS_s!"
dS = dS_n.ravel()

def sep_into_pos_and_neg_dist_and_norm(vals,dS):
    valspos, valsneg = np.copy(vals),-vals
    valspos[valspos < 0] = 0
    valsneg[valsneg < 0] = 0
    
    vals_sum = np.sum(valspos*dS+valsneg*dS)
    
    valspos = valspos/vals_sum
    valsneg = valsneg/vals_sum
    
    return valspos,valsneg


##############################
# Loop
# lists for holding CPCP for each bz value
cpcpn, cpcps = [], []
locns, locss = [], []
fig = plt.figure(1,figsize = (9, 9))

for tilt in tilts:#tilt = tilts[0]

    plt.clf()

    axes = [[Polarplot(plt.subplot2grid((91, 3), (j*30, i), rowspan = 30),
                          linestyle = ':', color = 'grey', minlat = minlat, linewidth = .9)
             for i in range(3) if (i, j) != (1, 1)]
             for j in range(3)]
    axdial = plt.subplot2grid((91, 3), (30, 1), rowspan = 30)
    axdial.set_aspect('equal')
    axdial.set_axis_off()
    # if withFAC:
    axcbar = plt.subplot2grid((91, 3), (90, 1))
    axinfo = plt.subplot2grid((91, 3), (90, 0))
    axinfo.set_axis_off()

    tmpcpcpn = []
    tmpcpcps = []
    tmplocn, tmplocs = [],[]
    for i in range(3):
        for j in range(3):
            if i == j == 1:
                continue # skip
            _ca = ca[(i,j)]
            By, Bz = np.sin(_ca * d2r) * B, np.cos(_ca * d2r) * B
            if _ca in [0, -90, 180, 90]:
                axdial.text(np.sin(_ca * d2r), np.cos(_ca * d2r),
                            str(_ca) + r'$^\circ$',
                            # rotation = -_ca,
                            ha = 'center', va = 'center')
                axdial.plot([.75*np.sin(_ca * d2r), .85*np.sin(_ca * d2r)],
                            [.75*np.cos(_ca * d2r), .85*np.cos(_ca * d2r)],
                            color = 'lightgrey')

            if (i, j) == (1, 2):
                j = 1
            swipe_n.update_model(v,  By, Bz,  tilt, f107val)
            swipe_s.update_model(v, -By, Bz, -tilt, f107val)


            SigmaHN, SigmaPN, maskN = self.get_conductances(mlat_n, mlt_n)
            SigmaHS, SigmaPS, maskS = self.get_conductances(mlat_s, mlt_s)

            if conductance == 'p':
                phin = SigmaPN
                phis = SigmaPS
            elif conductance == 'h':
                phin = SigmaHN
                phis = SigmaHS
            elif conductance == 'c':
                phin = SigmaPN
                phis = SigmaPS

            phin = phin.ravel()
            phis = phis.ravel()

            phinm = np.ma.masked_array(phin,mask=maskN)
            phism = np.ma.masked_array(phis,mask=maskS)

            filledn = axes[i][j].filled_cells(mlat_n, mlt_n, dr, np.split(mltres,2)[0], SigmaHN,
                                resolution = 10, crange = None, levels = sighlevels, bgcolor = 'lightgray',
                                verbose = False, **kwargs)

            contn = axes[i][j].contourf(mlat_n, mlt_n, phin, levels = potlevels, cmap = cmap, extend = extend)
            conts = axes[i][j].contour (mlat_s, mlt_s, phis, levels = potlevels, colors = 'k' , extend = extend, linewidths = .4)

            if doCalcBhattacharyya:
                # Separate into pos and neg current distributions
                psipos_n, psineg_n = np.copy(phin),-phin
                psipos_n[psipos_n < 0] = 0
                psineg_n[psineg_n < 0] = 0
            
                psipos_s, psineg_s = np.copy(phis),-phis
                psipos_s[psipos_s < 0] = 0
                psineg_s[psineg_s < 0] = 0
                
                phin_sum = np.sum(psipos_n*dS+psineg_n*dS)
                phis_sum = np.sum(psipos_s*dS+psineg_s*dS)
            
                # Normalize pos and neg current distributions
                psipos_n, psineg_n = sep_into_pos_and_neg_dist_and_norm(phin,dS)
                psipos_s, psineg_s = sep_into_pos_and_neg_dist_and_norm(phis,dS)
            
                # Calculate integrands for Bhattacharya coefficient
                facposintegrand = np.sqrt(psipos_n*psipos_s)
                facnegintegrand = np.sqrt(psineg_n*psineg_s)
                
                facBhatcoeff = np.sum(facposintegrand*dS+facnegintegrand*dS)

                showstring += "\n"+r"$BC = $"+f"{facBhatcoeff:.2f}"

    axdial.text(0, 0, titstr+"\nconductance", ha = 'center', va = 'center', size = 14)

    plt.subplots_adjust(hspace = .01, wspace = .01, left = .01, right = .99, bottom = .05, top = .99)

    axdial.set_xlim(-1.2, 1.2)
    axdial.set_ylim(-1.2, 1.2)

    cb = plt.colorbar(contn,cax=axcbar,orientation='horizontal')

    axcbar.set_yticks([])
    axcbar.set_xlabel('mho')

    axcbar.set_xticks([-15,-10,-5,0,5,10,15])

    axinfo.text(axinfo.get_xlim()[0], axinfo.get_ylim()[0], '$B_T$ = %s nT, $v$ = %s km/s,\n F$_{10.7}$ = %s, TILT $= %s^\circ$' % (B, v, f107val, tilt), ha = 'left', va = 'top', size = 12)

    plt.savefig(plotdir+'swipe_sigma'+conductance+'_ns_comparison_' + str(tilt) + '_deg_tilt.png', dpi = 300)

print("Save data ...")
plt.show()
